package com.netpotlet.cove.el;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;
import javax.faces.el.VariableResolver;
import javax.faces.el.EvaluationException;

import org.apache.commons.el.BinaryOperatorExpression;
import org.apache.commons.el.ComplexValue;
import org.apache.commons.el.ConditionalExpression;
import org.apache.commons.el.Constants;
import org.apache.commons.el.Expression;
import org.apache.commons.el.ExpressionString;
import org.apache.commons.el.FunctionInvocation;
import org.apache.commons.el.Literal;
import org.apache.commons.el.NamedValue;
import org.apache.commons.el.PropertySuffix;
import org.apache.commons.el.UnaryOperatorExpression;

public class CoveElEvaluator {
    private VariableResolver variableResolver;

    public CoveElEvaluator(VariableResolver resolver) {
        this.variableResolver = resolver;
    }

    public String evaluate(FacesContext context, Object object)
        throws
            EvaluationException,
            IllegalArgumentException,
            NoSuchMethodException,
            IllegalAccessException,
            InvocationTargetException {
        StringBuffer sb = new StringBuffer();
        if (object == null) {
            throw new EvaluationException(Constants.NULL_EXPRESSION_STRING);
        }

        if (object instanceof String) {
            sb.append((String) object);
        } else if (object instanceof ExpressionString) {
            Object[] objects = ((ExpressionString) object).getElements();
            for (int i = 0; i < objects.length; i++) {
                sb.append(evaluate(context, objects[i]));
            }
        } else if (object instanceof Expression) {
            if (object instanceof ComplexValue) {
                ComplexValue cValue = (ComplexValue) object;
                String prefix = cValue.getPrefix().getExpressionString();
                Object implicitObject =
                    variableResolver.resolveVariable(context, prefix);
                if (implicitObject == null) {
                    String valueRef = cValue.getExpressionString();
                    sb.append(getReferencedValue(context, valueRef));
                } else {
                    List suffixes = cValue.getSuffixes();
                    sb.append(getVariable(implicitObject, suffixes));
                }
            } else if (object instanceof NamedValue) {
                sb.append(((NamedValue) object).getExpressionString());
            } else if (object instanceof ConditionalExpression) {
                String dummy = "condition";
            } else if (object instanceof FunctionInvocation) {
                String dummy = "function";
            } else if (object instanceof Literal) {
                String dummy = "literal";
            } else if (object instanceof BinaryOperatorExpression) {
                String dummy = "binaryOp";
                //do i need to implement ?
            } else if (object instanceof UnaryOperatorExpression) {
                String dummy = "UnaryOp";
                //do i need to implement ?
            }
        }
        return new String(sb);
    }

    private String getReferencedValue(FacesContext context, String valueRef) {
        ValueBinding valueBinding = CoveElUtil.getValueBinding(valueRef);
        Object obj = valueBinding.getValue(context);
        if (obj instanceof String) {
            return (String) obj;
        }
        return null;
    }

    private String getVariable(Object object, List sName)
        throws
            NoSuchMethodException,
            IllegalArgumentException,
            IllegalAccessException,
            InvocationTargetException {
        if ((object == null) || (sName == null))
            return null;
        if (object instanceof Map) {
            Map map = (Map) object;
            PropertySuffix suffix = (PropertySuffix) sName.get(0);
            Object ret = map.get(suffix.getName());
            if (ret == null) {
                return "";
            } else {
                return (String) ret;
            }
        } else if (object instanceof ExternalContext) {
            Object from = object;
            for (int i = 0; i < sName.size(); i++) {
                PropertySuffix suffix = (PropertySuffix) sName.get(i);
                String methodName = getMethodName(suffix.getName());
                from = execute(from, methodName);
            }
            return (String) from;
        }
        return null;
    }

    private String getMethodName(String str) {
        StringBuffer sb = new StringBuffer("get");
        String firstLetter = str.substring(0, 1);
        sb.append(firstLetter.toUpperCase());
        sb.append(str.substring(1));
        return new String(sb);
    }

    private Object execute(Object object, String methodName)
        throws
            SecurityException,
            NoSuchMethodException,
            IllegalArgumentException,
            IllegalAccessException,
            InvocationTargetException {
        Method method = object.getClass().getMethod(methodName, null);
        return method.invoke(object, null);
    }
}