/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.netpotlet.xul.element;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>XulStartWindow</b> is generated from language.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="window">
 *       <attribute name="id">
 *         <data type="NCName"/>
 *       </attribute>
 *       <attribute name="title"/>
 *       <element name="groupbox">
 *         <attribute name="align">
 *           <data type="NCName"/>
 *         </attribute>
 *         <attribute name="flex">
 *           <data type="integer"/>
 *         </attribute>
 *         <attribute name="id">
 *           <data type="NCName"/>
 *         </attribute>
 *         <attribute name="style"/>
 *         <element name="caption">
 *           <attribute name="id">
 *             <data type="NCName"/>
 *           </attribute>
 *           <attribute name="label"/>
 *         </element>
 *         <element name="grid">
 *           <attribute name="id">
 *             <data type="NCName"/>
 *           </attribute>
 *           <attribute name="style"/>
 *           <element name="columns">
 *             <attribute name="id">
 *               <data type="NCName"/>
 *             </attribute>
 *             <oneOrMore>
 *               <element name="column">
 *                 <attribute name="id">
 *                   <data type="NCName"/>
 *                 </attribute>
 *                 <attribute name="width">
 *                   <data type="integer"/>
 *                 </attribute>
 *               </element>
 *             </oneOrMore>
 *           </element>
 *           <element name="rows">
 *             <attribute name="id">
 *               <data type="NCName"/>
 *             </attribute>
 *             <element name="row">
 *               <attribute name="id">
 *                 <data type="NCName"/>
 *               </attribute>
 *               <oneOrMore>
 *                 <choice>
 *                   <element name="button">
 *                     <optional>
 *                       <attribute name="checked">
 *                         <data type="boolean"/>
 *                       </attribute>
 *                     </optional>
 *                     <attribute name="id">
 *                       <data type="NCName"/>
 *                     </attribute>
 *                     <attribute name="image">
 *                       <data type="NCName"/>
 *                     </attribute>
 *                     <attribute name="label">
 *                       <data type="NCName"/>
 *                     </attribute>
 *                     <attribute name="orient">
 *                       <data type="NCName"/>
 *                     </attribute>
 *                   </element>
 *                   <element name="spacer">
 *                     <attribute name="id">
 *                       <data type="NCName"/>
 *                     </attribute>
 *                   </element>
 *                 </choice>
 *               </oneOrMore>
 *             </element>
 *           </element>
 *         </element>
 *       </element>
 *     </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element name="window"&gt;
 *       &lt;attribute name="id"&gt;
 *         &lt;data type="NCName"/&gt;
 *       &lt;/attribute&gt;
 *       &lt;attribute name="title"/&gt;
 *       &lt;element name="groupbox"&gt;
 *         &lt;attribute name="align"&gt;
 *           &lt;data type="NCName"/&gt;
 *         &lt;/attribute&gt;
 *         &lt;attribute name="flex"&gt;
 *           &lt;data type="integer"/&gt;
 *         &lt;/attribute&gt;
 *         &lt;attribute name="id"&gt;
 *           &lt;data type="NCName"/&gt;
 *         &lt;/attribute&gt;
 *         &lt;attribute name="style"/&gt;
 *         &lt;element name="caption"&gt;
 *           &lt;attribute name="id"&gt;
 *             &lt;data type="NCName"/&gt;
 *           &lt;/attribute&gt;
 *           &lt;attribute name="label"/&gt;
 *         &lt;/element&gt;
 *         &lt;element name="grid"&gt;
 *           &lt;attribute name="id"&gt;
 *             &lt;data type="NCName"/&gt;
 *           &lt;/attribute&gt;
 *           &lt;attribute name="style"/&gt;
 *           &lt;element name="columns"&gt;
 *             &lt;attribute name="id"&gt;
 *               &lt;data type="NCName"/&gt;
 *             &lt;/attribute&gt;
 *             &lt;oneOrMore&gt;
 *               &lt;element name="column"&gt;
 *                 &lt;attribute name="id"&gt;
 *                   &lt;data type="NCName"/&gt;
 *                 &lt;/attribute&gt;
 *                 &lt;attribute name="width"&gt;
 *                   &lt;data type="integer"/&gt;
 *                 &lt;/attribute&gt;
 *               &lt;/element&gt;
 *             &lt;/oneOrMore&gt;
 *           &lt;/element&gt;
 *           &lt;element name="rows"&gt;
 *             &lt;attribute name="id"&gt;
 *               &lt;data type="NCName"/&gt;
 *             &lt;/attribute&gt;
 *             &lt;element name="row"&gt;
 *               &lt;attribute name="id"&gt;
 *                 &lt;data type="NCName"/&gt;
 *               &lt;/attribute&gt;
 *               &lt;oneOrMore&gt;
 *                 &lt;choice&gt;
 *                   &lt;element name="button"&gt;
 *                     &lt;optional&gt;
 *                       &lt;attribute name="checked"&gt;
 *                         &lt;data type="boolean"/&gt;
 *                       &lt;/attribute&gt;
 *                     &lt;/optional&gt;
 *                     &lt;attribute name="id"&gt;
 *                       &lt;data type="NCName"/&gt;
 *                     &lt;/attribute&gt;
 *                     &lt;attribute name="image"&gt;
 *                       &lt;data type="NCName"/&gt;
 *                     &lt;/attribute&gt;
 *                     &lt;attribute name="label"&gt;
 *                       &lt;data type="NCName"/&gt;
 *                     &lt;/attribute&gt;
 *                     &lt;attribute name="orient"&gt;
 *                       &lt;data type="NCName"/&gt;
 *                     &lt;/attribute&gt;
 *                   &lt;/element&gt;
 *                   &lt;element name="spacer"&gt;
 *                     &lt;attribute name="id"&gt;
 *                       &lt;data type="NCName"/&gt;
 *                     &lt;/attribute&gt;
 *                   &lt;/element&gt;
 *                 &lt;/choice&gt;
 *               &lt;/oneOrMore&gt;
 *             &lt;/element&gt;
 *           &lt;/element&gt;
 *         &lt;/element&gt;
 *       &lt;/element&gt;
 *     &lt;/element&gt;</pre>
 *
 * @version language.rng (Mon May 31 15:52:18 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class XulStartWindow implements java.io.Serializable, Cloneable, IRNSContainer, IRVisitable, IRNode {
    private RNSContext rNSContext_ = new RNSContext(this, "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
    private String id_;
    private String title_;
    private XulStartWindowGroupbox startWindowGroupbox_;
    private IRNode parentRNode_;

    /**
     * Creates a <code>XulStartWindow</code>.
     *
     */
    public XulStartWindow() {
        id_ = "";
        title_ = "";
    }

    /**
     * Creates a <code>XulStartWindow</code>.
     *
     * @param source
     */
    public XulStartWindow(XulStartWindow source) {
        setup(source);
    }

    /**
     * Creates a <code>XulStartWindow</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public XulStartWindow(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>XulStartWindow</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public XulStartWindow(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>XulStartWindow</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public XulStartWindow(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>XulStartWindow</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulStartWindow(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>XulStartWindow</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulStartWindow(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>XulStartWindow</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulStartWindow(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>XulStartWindow</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulStartWindow(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>XulStartWindow</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulStartWindow(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>XulStartWindow</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulStartWindow(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>XulStartWindow</code> by the XulStartWindow <code>source</code>.
     *
     * @param source
     */
    public void setup(XulStartWindow source) {
        int size;
        id_ = source.id_;
        title_ = source.title_;
        if (source.startWindowGroupbox_ != null) {
            setStartWindowGroupbox((XulStartWindowGroupbox)source.getStartWindowGroupbox().clone());
        }
    }

    /**
     * Initializes the <code>XulStartWindow</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>XulStartWindow</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>XulStartWindow</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        rNSContext_.declareNamespace(element);
        id_ = URelaxer.getAttributePropertyAsString(element, "id");
        title_ = URelaxer.getAttributePropertyAsString(element, "title");
        setStartWindowGroupbox(new XulStartWindowGroupbox(stack));
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new XulStartWindow((XulStartWindow)this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "window");
        rNSContext_.setupNamespace(element);
        int size;
        if (this.id_ != null) {
            URelaxer.setAttributePropertyByString(element, "id", this.id_);
        }
        if (this.title_ != null) {
            URelaxer.setAttributePropertyByString(element, "title", this.title_);
        }
        this.startWindowGroupbox_.makeElement(element);
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>XulStartWindow</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>XulStartWindow</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulStartWindow</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulStartWindow</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulStartWindow</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulStartWindow</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the RNSContext property <b>RNSContext</b>.
     *
     * @return RNSContext
     */
    public RNSContext rGetRNSContext() {
        return (rNSContext_);
    }

    /**
     * Sets the RNSContext property <b>RNSContext</b>.
     *
     * @param rNSContext
     */
    public void rSetRNSContext(RNSContext rNSContext) {
        this.rNSContext_ = rNSContext;
    }

    /**
     * Gets the String property <b>id</b>.
     *
     * @return String
     */
    public String getId() {
        return (id_);
    }

    /**
     * Sets the String property <b>id</b>.
     *
     * @param id
     */
    public void setId(String id) {
        this.id_ = id;
    }

    /**
     * Gets the String property <b>title</b>.
     *
     * @return String
     */
    public String getTitle() {
        return (title_);
    }

    /**
     * Sets the String property <b>title</b>.
     *
     * @param title
     */
    public void setTitle(String title) {
        this.title_ = title;
    }

    /**
     * Gets the XulStartWindowGroupbox property <b>startWindowGroupbox</b>.
     *
     * @return XulStartWindowGroupbox
     */
    public XulStartWindowGroupbox getStartWindowGroupbox() {
        return (startWindowGroupbox_);
    }

    /**
     * Sets the XulStartWindowGroupbox property <b>startWindowGroupbox</b>.
     *
     * @param startWindowGroupbox
     */
    public void setStartWindowGroupbox(XulStartWindowGroupbox startWindowGroupbox) {
        this.startWindowGroupbox_ = startWindowGroupbox;
        if (startWindowGroupbox != null) {
            startWindowGroupbox.rSetParentRNode(this);
        }
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.append("<");
        URelaxer.makeQName(prefix, "window", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (id_ != null) {
            buffer.append(" ");
            buffer.append("id");
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.append("\"");
        }
        if (title_ != null) {
            buffer.append(" ");
            buffer.append("title");
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getTitle())));
            buffer.append("\"");
        }
        buffer.append(">");
        startWindowGroupbox_.makeTextElement(buffer);
        buffer.append("</");
        URelaxer.makeQName(prefix, "window", buffer);
        buffer.append(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.write("<");
        URelaxer.makeQName(prefix, "window", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (id_ != null) {
            buffer.write(" ");
            buffer.write("id");
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.write("\"");
        }
        if (title_ != null) {
            buffer.write(" ");
            buffer.write("title");
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getTitle())));
            buffer.write("\"");
        }
        buffer.write(">");
        startWindowGroupbox_.makeTextElement(buffer);
        buffer.write("</");
        URelaxer.makeQName(prefix, "window", buffer);
        buffer.write(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.print("<");
        URelaxer.makeQName(prefix, "window", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (id_ != null) {
            buffer.print(" ");
            buffer.print("id");
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.print("\"");
        }
        if (title_ != null) {
            buffer.print(" ");
            buffer.print("title");
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getTitle())));
            buffer.print("\"");
        }
        buffer.print(">");
        startWindowGroupbox_.makeTextElement(buffer);
        buffer.print("</");
        URelaxer.makeQName(prefix, "window", buffer);
        buffer.print(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getIdAsString() {
        return (URelaxer.getString(getId()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getTitleAsString() {
        return (URelaxer.getString(getTitle()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setIdByString(String string) {
        setId(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setTitleByString(String string) {
        setTitle(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Accepts the Visitor for enter behavior.
     *
     * @param visitor
     * @return boolean
     */
    public boolean enter(IRVisitor visitor) {
        return (visitor.enter(this));
    }

    /**
     * Accepts the Visitor for leave behavior.
     *
     * @param visitor
     */
    public void leave(IRVisitor visitor) {
        visitor.leave(this);
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        if (startWindowGroupbox_ != null) {
            classNodes.add(startWindowGroupbox_);
        }
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>XulStartWindow</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer2.isTargetElement(element, "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "window")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        if (!URelaxer.hasAttributeHungry(target, "id")) {
            return (false);
        }
        $match$ = true;
        if (!URelaxer.hasAttributeHungry(target, "title")) {
            return (false);
        }
        $match$ = true;
        if (!XulStartWindowGroupbox.isMatchHungry(target)) {
            return (false);
        }
        $match$ = true;
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulStartWindow</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulStartWindow</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
