/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.netpotlet.xul.element;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>XulStartWindowGroupbox</b> is generated from language.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="groupbox">
 *         <attribute name="align">
 *           <data type="NCName"/>
 *         </attribute>
 *         <attribute name="flex">
 *           <data type="integer"/>
 *         </attribute>
 *         <attribute name="id">
 *           <data type="NCName"/>
 *         </attribute>
 *         <attribute name="style"/>
 *         <element name="caption">
 *           <attribute name="id">
 *             <data type="NCName"/>
 *           </attribute>
 *           <attribute name="label"/>
 *         </element>
 *         <element name="grid">
 *           <attribute name="id">
 *             <data type="NCName"/>
 *           </attribute>
 *           <attribute name="style"/>
 *           <element name="columns">
 *             <attribute name="id">
 *               <data type="NCName"/>
 *             </attribute>
 *             <oneOrMore>
 *               <element name="column">
 *                 <attribute name="id">
 *                   <data type="NCName"/>
 *                 </attribute>
 *                 <attribute name="width">
 *                   <data type="integer"/>
 *                 </attribute>
 *               </element>
 *             </oneOrMore>
 *           </element>
 *           <element name="rows">
 *             <attribute name="id">
 *               <data type="NCName"/>
 *             </attribute>
 *             <element name="row">
 *               <attribute name="id">
 *                 <data type="NCName"/>
 *               </attribute>
 *               <oneOrMore>
 *                 <choice>
 *                   <element name="button">
 *                     <optional>
 *                       <attribute name="checked">
 *                         <data type="boolean"/>
 *                       </attribute>
 *                     </optional>
 *                     <attribute name="id">
 *                       <data type="NCName"/>
 *                     </attribute>
 *                     <attribute name="image">
 *                       <data type="NCName"/>
 *                     </attribute>
 *                     <attribute name="label">
 *                       <data type="NCName"/>
 *                     </attribute>
 *                     <attribute name="orient">
 *                       <data type="NCName"/>
 *                     </attribute>
 *                   </element>
 *                   <element name="spacer">
 *                     <attribute name="id">
 *                       <data type="NCName"/>
 *                     </attribute>
 *                   </element>
 *                 </choice>
 *               </oneOrMore>
 *             </element>
 *           </element>
 *         </element>
 *       </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element name="groupbox"&gt;
 *         &lt;attribute name="align"&gt;
 *           &lt;data type="NCName"/&gt;
 *         &lt;/attribute&gt;
 *         &lt;attribute name="flex"&gt;
 *           &lt;data type="integer"/&gt;
 *         &lt;/attribute&gt;
 *         &lt;attribute name="id"&gt;
 *           &lt;data type="NCName"/&gt;
 *         &lt;/attribute&gt;
 *         &lt;attribute name="style"/&gt;
 *         &lt;element name="caption"&gt;
 *           &lt;attribute name="id"&gt;
 *             &lt;data type="NCName"/&gt;
 *           &lt;/attribute&gt;
 *           &lt;attribute name="label"/&gt;
 *         &lt;/element&gt;
 *         &lt;element name="grid"&gt;
 *           &lt;attribute name="id"&gt;
 *             &lt;data type="NCName"/&gt;
 *           &lt;/attribute&gt;
 *           &lt;attribute name="style"/&gt;
 *           &lt;element name="columns"&gt;
 *             &lt;attribute name="id"&gt;
 *               &lt;data type="NCName"/&gt;
 *             &lt;/attribute&gt;
 *             &lt;oneOrMore&gt;
 *               &lt;element name="column"&gt;
 *                 &lt;attribute name="id"&gt;
 *                   &lt;data type="NCName"/&gt;
 *                 &lt;/attribute&gt;
 *                 &lt;attribute name="width"&gt;
 *                   &lt;data type="integer"/&gt;
 *                 &lt;/attribute&gt;
 *               &lt;/element&gt;
 *             &lt;/oneOrMore&gt;
 *           &lt;/element&gt;
 *           &lt;element name="rows"&gt;
 *             &lt;attribute name="id"&gt;
 *               &lt;data type="NCName"/&gt;
 *             &lt;/attribute&gt;
 *             &lt;element name="row"&gt;
 *               &lt;attribute name="id"&gt;
 *                 &lt;data type="NCName"/&gt;
 *               &lt;/attribute&gt;
 *               &lt;oneOrMore&gt;
 *                 &lt;choice&gt;
 *                   &lt;element name="button"&gt;
 *                     &lt;optional&gt;
 *                       &lt;attribute name="checked"&gt;
 *                         &lt;data type="boolean"/&gt;
 *                       &lt;/attribute&gt;
 *                     &lt;/optional&gt;
 *                     &lt;attribute name="id"&gt;
 *                       &lt;data type="NCName"/&gt;
 *                     &lt;/attribute&gt;
 *                     &lt;attribute name="image"&gt;
 *                       &lt;data type="NCName"/&gt;
 *                     &lt;/attribute&gt;
 *                     &lt;attribute name="label"&gt;
 *                       &lt;data type="NCName"/&gt;
 *                     &lt;/attribute&gt;
 *                     &lt;attribute name="orient"&gt;
 *                       &lt;data type="NCName"/&gt;
 *                     &lt;/attribute&gt;
 *                   &lt;/element&gt;
 *                   &lt;element name="spacer"&gt;
 *                     &lt;attribute name="id"&gt;
 *                       &lt;data type="NCName"/&gt;
 *                     &lt;/attribute&gt;
 *                   &lt;/element&gt;
 *                 &lt;/choice&gt;
 *               &lt;/oneOrMore&gt;
 *             &lt;/element&gt;
 *           &lt;/element&gt;
 *         &lt;/element&gt;
 *       &lt;/element&gt;</pre>
 *
 * @version language.rng (Mon May 31 15:52:18 JST 2004)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class XulStartWindowGroupbox implements java.io.Serializable, Cloneable, IRNSContainer, IRVisitable, IRNode {
    private RNSContext rNSContext_ = new RNSContext(this, "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
    private String align_;
    private java.math.BigInteger flex_;
    private String id_;
    private String style_;
    private XulStartWindowGroupboxCaption startWindowGroupboxCaption_;
    private XulStartWindowGroupboxGrid startWindowGroupboxGrid_;
    private IRNode parentRNode_;

    /**
     * Creates a <code>XulStartWindowGroupbox</code>.
     *
     */
    public XulStartWindowGroupbox() {
        align_ = "";
        flex_ = java.math.BigInteger.ZERO;
        id_ = "";
        style_ = "";
    }

    /**
     * Creates a <code>XulStartWindowGroupbox</code>.
     *
     * @param source
     */
    public XulStartWindowGroupbox(XulStartWindowGroupbox source) {
        setup(source);
    }

    /**
     * Creates a <code>XulStartWindowGroupbox</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public XulStartWindowGroupbox(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>XulStartWindowGroupbox</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public XulStartWindowGroupbox(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>XulStartWindowGroupbox</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public XulStartWindowGroupbox(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>XulStartWindowGroupbox</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulStartWindowGroupbox(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>XulStartWindowGroupbox</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulStartWindowGroupbox(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>XulStartWindowGroupbox</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulStartWindowGroupbox(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>XulStartWindowGroupbox</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulStartWindowGroupbox(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>XulStartWindowGroupbox</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulStartWindowGroupbox(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>XulStartWindowGroupbox</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public XulStartWindowGroupbox(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>XulStartWindowGroupbox</code> by the XulStartWindowGroupbox <code>source</code>.
     *
     * @param source
     */
    public void setup(XulStartWindowGroupbox source) {
        int size;
        align_ = source.align_;
        flex_ = source.flex_;
        id_ = source.id_;
        style_ = source.style_;
        if (source.startWindowGroupboxCaption_ != null) {
            setStartWindowGroupboxCaption((XulStartWindowGroupboxCaption)source.getStartWindowGroupboxCaption().clone());
        }
        if (source.startWindowGroupboxGrid_ != null) {
            setStartWindowGroupboxGrid((XulStartWindowGroupboxGrid)source.getStartWindowGroupboxGrid().clone());
        }
    }

    /**
     * Initializes the <code>XulStartWindowGroupbox</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>XulStartWindowGroupbox</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>XulStartWindowGroupbox</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        rNSContext_.declareNamespace(element);
        align_ = URelaxer.getAttributePropertyAsString(element, "align");
        flex_ = URelaxer.getAttributePropertyAsBigInteger(element, "flex");
        id_ = URelaxer.getAttributePropertyAsString(element, "id");
        style_ = URelaxer.getAttributePropertyAsString(element, "style");
        setStartWindowGroupboxCaption(new XulStartWindowGroupboxCaption(stack));
        setStartWindowGroupboxGrid(new XulStartWindowGroupboxGrid(stack));
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new XulStartWindowGroupbox((XulStartWindowGroupbox)this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "groupbox");
        rNSContext_.setupNamespace(element);
        int size;
        if (this.align_ != null) {
            URelaxer.setAttributePropertyByString(element, "align", this.align_);
        }
        if (this.flex_ != null) {
            URelaxer.setAttributePropertyByBigInteger(element, "flex", this.flex_);
        }
        if (this.id_ != null) {
            URelaxer.setAttributePropertyByString(element, "id", this.id_);
        }
        if (this.style_ != null) {
            URelaxer.setAttributePropertyByString(element, "style", this.style_);
        }
        this.startWindowGroupboxCaption_.makeElement(element);
        this.startWindowGroupboxGrid_.makeElement(element);
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>XulStartWindowGroupbox</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>XulStartWindowGroupbox</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulStartWindowGroupbox</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulStartWindowGroupbox</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulStartWindowGroupbox</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Initializes the <code>XulStartWindowGroupbox</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NAMESPACE_AWARE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the RNSContext property <b>RNSContext</b>.
     *
     * @return RNSContext
     */
    public RNSContext rGetRNSContext() {
        return (rNSContext_);
    }

    /**
     * Sets the RNSContext property <b>RNSContext</b>.
     *
     * @param rNSContext
     */
    public void rSetRNSContext(RNSContext rNSContext) {
        this.rNSContext_ = rNSContext;
    }

    /**
     * Gets the String property <b>align</b>.
     *
     * @return String
     */
    public String getAlign() {
        return (align_);
    }

    /**
     * Sets the String property <b>align</b>.
     *
     * @param align
     */
    public void setAlign(String align) {
        this.align_ = align;
    }

    /**
     * Gets the java.math.BigInteger property <b>flex</b>.
     *
     * @return java.math.BigInteger
     */
    public java.math.BigInteger getFlex() {
        return (flex_);
    }

    /**
     * Sets the java.math.BigInteger property <b>flex</b>.
     *
     * @param flex
     */
    public void setFlex(java.math.BigInteger flex) {
        this.flex_ = flex;
    }

    /**
     * Gets the String property <b>id</b>.
     *
     * @return String
     */
    public String getId() {
        return (id_);
    }

    /**
     * Sets the String property <b>id</b>.
     *
     * @param id
     */
    public void setId(String id) {
        this.id_ = id;
    }

    /**
     * Gets the String property <b>style</b>.
     *
     * @return String
     */
    public String getStyle() {
        return (style_);
    }

    /**
     * Sets the String property <b>style</b>.
     *
     * @param style
     */
    public void setStyle(String style) {
        this.style_ = style;
    }

    /**
     * Gets the XulStartWindowGroupboxCaption property <b>startWindowGroupboxCaption</b>.
     *
     * @return XulStartWindowGroupboxCaption
     */
    public XulStartWindowGroupboxCaption getStartWindowGroupboxCaption() {
        return (startWindowGroupboxCaption_);
    }

    /**
     * Sets the XulStartWindowGroupboxCaption property <b>startWindowGroupboxCaption</b>.
     *
     * @param startWindowGroupboxCaption
     */
    public void setStartWindowGroupboxCaption(XulStartWindowGroupboxCaption startWindowGroupboxCaption) {
        this.startWindowGroupboxCaption_ = startWindowGroupboxCaption;
        if (startWindowGroupboxCaption != null) {
            startWindowGroupboxCaption.rSetParentRNode(this);
        }
    }

    /**
     * Gets the XulStartWindowGroupboxGrid property <b>startWindowGroupboxGrid</b>.
     *
     * @return XulStartWindowGroupboxGrid
     */
    public XulStartWindowGroupboxGrid getStartWindowGroupboxGrid() {
        return (startWindowGroupboxGrid_);
    }

    /**
     * Sets the XulStartWindowGroupboxGrid property <b>startWindowGroupboxGrid</b>.
     *
     * @param startWindowGroupboxGrid
     */
    public void setStartWindowGroupboxGrid(XulStartWindowGroupboxGrid startWindowGroupboxGrid) {
        this.startWindowGroupboxGrid_ = startWindowGroupboxGrid;
        if (startWindowGroupboxGrid != null) {
            startWindowGroupboxGrid.rSetParentRNode(this);
        }
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.append("<");
        URelaxer.makeQName(prefix, "groupbox", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (align_ != null) {
            buffer.append(" ");
            buffer.append("align");
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getAlign())));
            buffer.append("\"");
        }
        if (flex_ != null) {
            buffer.append(" ");
            buffer.append("flex");
            buffer.append("=\"");
            buffer.append(URelaxer.getString(getFlex()));
            buffer.append("\"");
        }
        if (id_ != null) {
            buffer.append(" ");
            buffer.append("id");
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.append("\"");
        }
        if (style_ != null) {
            buffer.append(" ");
            buffer.append("style");
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getStyle())));
            buffer.append("\"");
        }
        buffer.append(">");
        startWindowGroupboxCaption_.makeTextElement(buffer);
        startWindowGroupboxGrid_.makeTextElement(buffer);
        buffer.append("</");
        URelaxer.makeQName(prefix, "groupbox", buffer);
        buffer.append(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.write("<");
        URelaxer.makeQName(prefix, "groupbox", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (align_ != null) {
            buffer.write(" ");
            buffer.write("align");
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getAlign())));
            buffer.write("\"");
        }
        if (flex_ != null) {
            buffer.write(" ");
            buffer.write("flex");
            buffer.write("=\"");
            buffer.write(URelaxer.getString(getFlex()));
            buffer.write("\"");
        }
        if (id_ != null) {
            buffer.write(" ");
            buffer.write("id");
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.write("\"");
        }
        if (style_ != null) {
            buffer.write(" ");
            buffer.write("style");
            buffer.write("=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getStyle())));
            buffer.write("\"");
        }
        buffer.write(">");
        startWindowGroupboxCaption_.makeTextElement(buffer);
        startWindowGroupboxGrid_.makeTextElement(buffer);
        buffer.write("</");
        URelaxer.makeQName(prefix, "groupbox", buffer);
        buffer.write(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        String prefix = rNSContext_.getPrefixByUri("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.print("<");
        URelaxer.makeQName(prefix, "groupbox", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (align_ != null) {
            buffer.print(" ");
            buffer.print("align");
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getAlign())));
            buffer.print("\"");
        }
        if (flex_ != null) {
            buffer.print(" ");
            buffer.print("flex");
            buffer.print("=\"");
            buffer.print(URelaxer.getString(getFlex()));
            buffer.print("\"");
        }
        if (id_ != null) {
            buffer.print(" ");
            buffer.print("id");
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.print("\"");
        }
        if (style_ != null) {
            buffer.print(" ");
            buffer.print("style");
            buffer.print("=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getStyle())));
            buffer.print("\"");
        }
        buffer.print(">");
        startWindowGroupboxCaption_.makeTextElement(buffer);
        startWindowGroupboxGrid_.makeTextElement(buffer);
        buffer.print("</");
        URelaxer.makeQName(prefix, "groupbox", buffer);
        buffer.print(">");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getAlignAsString() {
        return (URelaxer.getString(getAlign()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getFlexAsString() {
        return (URelaxer.getString(getFlex()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getIdAsString() {
        return (URelaxer.getString(getId()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getStyleAsString() {
        return (URelaxer.getString(getStyle()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setAlignByString(String string) {
        setAlign(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setFlexByString(String string) {
        setFlex(new java.math.BigInteger(string));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setIdByString(String string) {
        setId(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setStyleByString(String string) {
        setStyle(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Accepts the Visitor for enter behavior.
     *
     * @param visitor
     * @return boolean
     */
    public boolean enter(IRVisitor visitor) {
        return (visitor.enter(this));
    }

    /**
     * Accepts the Visitor for leave behavior.
     *
     * @param visitor
     */
    public void leave(IRVisitor visitor) {
        visitor.leave(this);
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        if (startWindowGroupboxCaption_ != null) {
            classNodes.add(startWindowGroupboxCaption_);
        }
        if (startWindowGroupboxGrid_ != null) {
            classNodes.add(startWindowGroupboxGrid_);
        }
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>XulStartWindowGroupbox</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer2.isTargetElement(element, "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "groupbox")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        if (!URelaxer.hasAttributeHungry(target, "align")) {
            return (false);
        }
        $match$ = true;
        if (!URelaxer.hasAttributeHungry(target, "flex")) {
            return (false);
        }
        $match$ = true;
        if (!URelaxer.hasAttributeHungry(target, "id")) {
            return (false);
        }
        $match$ = true;
        if (!URelaxer.hasAttributeHungry(target, "style")) {
            return (false);
        }
        $match$ = true;
        if (!XulStartWindowGroupboxCaption.isMatchHungry(target)) {
            return (false);
        }
        $match$ = true;
        if (!XulStartWindowGroupboxGrid.isMatchHungry(target)) {
            return (false);
        }
        $match$ = true;
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulStartWindowGroupbox</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>XulStartWindowGroupbox</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
