package com.netpotlet.xul.renderkit;

import java.io.IOException;

import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import org.apache.commons.el.parser.ParseException;

import com.netpotlet.cove.el.CoveElResolver;
import com.netpotlet.xul.component.CaptionComponent;
import com.netpotlet.xul.element.RNSContext;
import com.netpotlet.xul.element.URelaxer;

public class CaptionRenderer extends Renderer {

    public CaptionRenderer() {
        super();
    }

    public void encodeBegin(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        CaptionComponent caption = (CaptionComponent) component;
        try {
            char[] encoded = getEncodedBeginString(caption);
            ResponseWriter writer = context.getResponseWriter();
            writer.writeText(encoded, 0, encoded.length);
        } catch (ParseException e) {
            throw new FacesException(e);
        }
    }

    private char[] getEncodedBeginString(CaptionComponent caption)
        throws ParseException {
        String mixed = caption.getLabel();
        String label = CoveElResolver.getResolver().resolveExpression(mixed);
        StringBuffer buffer = new StringBuffer();
        RNSContext rNSContext_ = caption.getRNSContext();
        String prefix =
            rNSContext_.getPrefixByUri(
                "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.append("<");
        URelaxer.makeQName(prefix, "caption", buffer);
        rNSContext_.makeNSMappings(buffer);
        if (caption.getId() != null) {
            buffer.append(" ");
            buffer.append("id");
            buffer.append("=\"");
            buffer.append(
                URelaxer.escapeAttrQuot(URelaxer.getString(caption.getId())));
            buffer.append("\"");
        }
        if (label != null) {
            buffer.append(" ");
            buffer.append("label");
            buffer.append("=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(label)));
            buffer.append("\"");
        }
        buffer.append(">");
        return (new String(buffer)).toCharArray();
    }

    public void encodeChildren(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
    }

    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        CaptionComponent caption = (CaptionComponent) component;
        char[] encoded = getEncodedEndString(caption);
        ResponseWriter writer = context.getResponseWriter();
        writer.writeText(encoded, 0, encoded.length);
    }

    private char[] getEncodedEndString(CaptionComponent caption) {
        StringBuffer buffer = new StringBuffer();
        RNSContext rNSContext_ = caption.getRNSContext();
        String prefix =
            rNSContext_.getPrefixByUri(
                "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
        buffer.append("</");
        URelaxer.makeQName(prefix, "caption", buffer);
        buffer.append(">");
        return (new String(buffer)).toCharArray();
    }
}
